#!/usr/bin/env bash
# This program starts a child process of model 3/4/5 alignment
# It can perform the following tasks:
# 1. From HMM to Model 3
# 2. Model 3 Iteration
# 3. Model 4 Iteration
# It takes  ? Parameters
echo "---------------------------------------------------------------"
echo "Usage of this script:"
echo "runModelOneChild.sh <config> <output_t> <output_a> <output_d> "
echo "    <output_n> <output_pz> <mode> <signal> <log> <prev_tmodel> "
echo "    <prev_amodel> [prev_dmodel] [prev_nmodel] [prev_pzmodel]"
echo "1  <config> is the configure template, "
echo "2  <output_t>  is the path of the difference file for t table"
echo "3  <output_a>  is the path of the count table for a table"
echo "4  <output_d>  is the path of the count table for d table"
echo "5  <output_n>  is the path of the count table for n table"
echo "6  <output_pz> is the path of the count for p0 and p1"
echo "7  <output_hmm> is the path of the count for hmm"
echo "8  <mode>      is the task to run "
echo "               0: HMM to Model 3"
echo "               1: Model 3 Iteration"
echo "               2: Model 3 To Model 4"
echo "               3: Model 4 Iteration"
echo "9  <signal>    is the file which I will touch when finished"
echo "10  <log>       is the log file which I record everything"
echo "11 <prev_tmodel> previous t model file "
echo "12 <prev_amodel> previous a model file "
echo "13 <prev_hmodel> previous hmm file "
echo "14 [prev_dmodel] previous d model file "
echo "15 [prev_nmodel] previous n model file "
echo "16 [prev_pmodel] previous p0 file "
echo "Note that other than HMM to Model 3, you must provide all the"
echo "optional model file from previous step"
echo "---------------------------------------------------------------"
echo 
echo 
	if [ $# -lt "12" ];
then
	echo " NO! I need your command, see above for detail"
	exit 1
fi

if [ -e $PWD/defines.inc ];
then
        source $PWD/defines.inc
else
	VITERBI_EXEC=/SMT/giza/view_source/newgiza/src/para/model345align
fi

if [ -x $VITERBI_EXEC ];
then
	echo " Using executable " $VITERBI_EXEC 
else
	echo " The execution file " $VITERBI_EXEC " cannot be executed dude"
	echo " set environment HMM_EXEC to get me run!"
	exit 1
fi

# For safety reason, we should verify all the parameters
# and give them some name
TEMPL=$1     # Template File
OTTABLE=$2   # Output of TTable (Difference)
OATABLE=$3   # Output of ATable (Count)
ODTABLE=$4   # Output of DTable (Count)
ONTABLE=$5   # Output of NTable (Count)
OZTABLE=$6   # Output of p0,p1  (Count)
OHTABLE=$7	 # Output of HMMTable (Count, alpha, beta)
MODE=$8      # Running mode
SIGFILE=$9   # The signal to fire
LOGFILE=$10   # The log file
PTTABLE=${11}  # Previous ttable
PATABLE=${12}  # Previous a atable
PHTABLE=${13} # Previous hmm table

echo $TEMPL
# Start checking
if [ -r ${TEMPL} ]; then
	echo "Using template file " $TEMPL
else
	echo "Cannot find template file " $TEMPL
	exit 1	
fi

if [ -r $PTTABLE ]; then
	echo "Using previous t table file " $PTTABLE
else
	echo "Cannot find previous t table file " $PTTABLE
	exit 1
fi

if [ -r $PATABLE ]; then
	echo "Using previous a table file " $PATABLE
else
	echo "Cannot find previous a table file " $PATABLE
	exit 1
fi

if [ -e $PHTABLE ] && [ -e ${PHTABLE}.alpha ] && [ -e ${PHTABLE}.beta ]; then
	echo "Using previous hmm table file " $PPTABLE ,  ${PHTABLE}.alpha and ${PHTABLE}.beta
else
	echo "Cannot find previous hmm table file " $PPTABLE or ${PHTABLE}.alpha or ${PHTABLE}.beta
	exit 1
fi

if [ $MODE -gt "0" ];
then
	if [ $# -lt 14 ];
	then
		echo "You must provide models from last iteration"
		exit 1
	fi
	PDTABLE=${14} # Previous d table
	PNTABLE=${15} # Previous n table
	PPTABLE=${16} # Previous p0 table
	if [ -e $PDTABLE ]; then
		echo "Using previous d table file " $PDTABLE
	else
		echo "Cannot find previous d table file " $PDTABLE
		exit 1
	fi
	if [ -e $PNTABLE ]; then
		echo "Using previous n table file " $PNTABLE
	else
		echo "Cannot find previous n table file " $PNTABLE
		exit 1
	fi	
	if [ -e $PPTABLE ]; then
		echo "Using previous p table file " $PPTABLE
	else
		echo "Cannot find previous p table file " $PPTABLE
		exit 1
	fi	
fi


# Here is the configure file we are going to write
CONF=/tmp/`basename ${TEMPL}`.$$.${MODE}.345.gizaconf
# We need to test whether the environment is OK
if [ -r ${TEMPL} ]; # is template exists?
then
	echo "... Checking template"
	echo " GOOD, I am now copying the template to where we will start"
	cat ${TEMPL} > $CONF
	# test ?
	echo " The initial configure is written to file " $CONF
else
	echo " NO! The template cannot be read! "
	exit 1
fi

echo "prevtmodel " $PTTABLE >> $CONF
echo "prevamodel " $PATABLE >> $CONF
echo "prevhmodel " $PHTABLE >> $CONF
if [ $MODE != "0" ]; 
then
	echo "prevdmodel " $PDTABLE >> $CONF
	echo "prevnmodel " $PNTABLE >> $CONF
	echo "prevp0 " $PPTABLE >> $CONF
fi

echo "clv " $MODE >> $CONF
echo "outputdiff"  $OTTABLE >> $CONF
echo "outputacounttable" $OATABLE >> $CONF
echo "outputdcounttable" $ODTABLE >> $CONF
echo "outputncounttable" $ONTABLE >> $CONF
echo "outputpzcounttable" $OZTABLE >> $CONF
echo "outputhcounttable" $OHTABLE >> $CONF
echo "o" $ALIGN_DIR/`basename ${SIGFILE}` >> $CONF

# Now we have everything, start!
rm -f $SIGFILE
rm -f ${SIGFILE}.failed

rm -f ${SIGFILE}.starting
if $VITERBI_EXEC $CONF &> $LOGFILE ;
then
	echo " Yeah! we finish, please tell my mom at" $SIGFILE
	touch $SIGFILE
else
	echo " Something is wrong, see " $LOGFILE ", I wrote something"
	touch ${LOGFILE}.failed
	# When mother see this, do something to save me
fi

rm -f ${SIGFILE}.running




